;*********************************************************************
;	Steuereinheit zur Umsetzung/Auswertung LIN- Regensensor
;	VW 1K0 955 559 AH
;	Reiner Ost
;	Initial Version 08/2009
;	Diese Version   05/2017, umgebaut auf Tiny2313
;	
;	0.5 nderungen an den Frames, Auswertung des SF umgebaut
;	
;	0.6 Startframesteuerung und Mittelwertberechnung eingebaut 
;
;   Protokoll: A1@ fr Abfrage Lichtwert
;			   A2@ fr Abfrage Regenwert
;			   Antwort: einfacher Hex-Zahlenwert
;*********************************************************************

.include "tn2313def.inc"
.include "definitionen.inc"

;****************************** Vektor-Liste ************************

.CSEG

		rjmp    Reset		 
		
.ORG	URXCaddr
		rjmp	irq_rx

.ORG	OC0Aaddr
		rjmp	timer_irq	
		
;*********************************************************************
;  Initialisierungsroutinen
;*********************************************************************

.ORG INT_VECTORS_SIZE
; Hardware initialisieren

Reset:
		ldi     TEMP_0,low(RAMEND)          ; Stack an das interne Ram-Ende
        out     SPL,TEMP_0
		

; Port A *************************************************************
/*	
		ser		TEMP_0						; Port A als Ausgang				;   
		out     DDRA,TEMP_0
		out		PORTA,TEMP_0				; Push-Pull-Ausgang auf HIGH 
*/			
; Port B *************************************************************

		ldi		TEMP_0, 0x02				; Port B der 2. als Ausgang
		out     DDRB,TEMP_0
		ldi		TEMP_0, 0x01
        out     PORTB,TEMP_0 				; Pull-Up auf HIGH
		
; Port C *************************************************************
/*	
		ldi     TEMP_0, 0xF8		  		; Port C als Ausgang, die ersten 3 frei
		out		DDRC,TEMP_0					; 
        out     PORTC,TEMP_0 				; Push-Pull-Ausgang auf HIGH 
	
*/	
; Port D *************************************************************

		clr		TEMP_0
		ldi		TEMP_0,(1<<MCS) | (1<<LED)	; Wake up MCP201,			
        out     DDRD,TEMP_0
	//	out		PORTD, TEMP_0
		rcall   delay1ms
		sbi     PORTD, MCS					; init MCP wake up
; TIMER 0 - (8Bit) ***************************************************
		
		ldi		TEMP_0, (1<<WGM01)
		out		TCCR0A, TEMP_0
		ldi		TEMP_0, (1<<CS00)|(1<<CS02)			; CTC Mode 2, Prescaler 1024
		out		TCCR0B,TEMP_0
		ldi		TEMP_0,T0_COUNT
		out		OCR0A,TEMP_0
		ldi		TEMP_0,(1<<OCIE0A)
		out		TIMSK,TEMP_0


; UART 0 ***************************************************************

        ldi     TEMP_0,high(UART_BAUD_SELECT)
        out     UBRRH,TEMP_0
        ldi     TEMP_0,low(UART_BAUD_SELECT)
        out     UBRRL,TEMP_0
        ldi     TEMP_0,(1<<TXEN) | (1<<RXEN) | (1<<RXCIE)
        out     UCSRB,TEMP_0
        ldi     TEMP_0, (3<<UCSZ0)			; 8 Datenbits
        out     UCSRC,TEMP_0

;Software initialisieren*********************************************************************				 
		
		clr		TEMP_1
		clr		TEMP_2
		clr		TEMP_3
		clr		NULL
		clr		FLAGS
		sbr		FLAGS,(1<<START)
		ldi		TEMP_0,40
		mov		STARTCNT,TEMP_0
		clr		TEMP_0
		sei								; Interrups freigeben
//		load_p	Z,(2*text_reset)
//		rcall	uart_send_text												

;Programmstart***********************************************************

main:
		sbrc	FLAGS, LRX				; neuer Regen/Lichtwert
		rcall	auswert
		nop
		sbrc	FLAGS, MF				; Masterframe senden?
		rcall	h_send_mf
		nop
		sbrc	FLAGS, SF				; Slaveframe senden?
		rcall	h_send_sf
		nop
		sbrc	FLAGS, UF				; Wakeupframe senden?
		rcall	h_send_wf
		nop
		sbrc	FLAGS, FLAG_RX_OK		; Anforderung Wert
		rcall	senden
		nop

		rjmp	main


;*************************** IRQ - Routinen **************************
;*********************************************************************
;  IRQ TIMER
;*********************************************************************

timer_irq:
		push	TEMP_0
		in		SICH_SREG,SREG
		inc		COUNTER					; Time counter herhhen

		mov		TEMP_0,FLAGS			; Kopie holen
		andi	TEMP_0,0b11100000		; Frameflags isolieren
		brne	irq_fertig				; wenn Frame noch nicht abgearbeitet 
		tst		FRAMECNT				; Framecnt null?	
		breq 	zero
		cpi		FRAMECNT,0x09			; 10.Frame?		
		breq 	zero
		sbrc	FRAMECNT,0				; ungrade, also MF  
		sbr		FLAGS,(1<<MF)		
		sbrs	FRAMECNT,0				; grade, also SF
		sbr		FLAGS,(1<<SF)
		cpi		FRAMECNT,0x0A			; 10.Frame?
		brlo	irq_fertig
		rjmp	von_vorn

zero:
		
		sbr		FLAGS,(1<<UF)			; wakeup frame senden
		cbi		PORTD, LED
		rjmp	irq_fertig

von_vorn:
		clr     FRAMECNT
		sbi		PORTD, LED
irq_fertig:
		pop		TEMP_0
		out		SREG,SICH_SREG
		reti		


;*********************************************************************
;  IRQ RX
;*********************************************************************
irq_rx:
		in		SICH_SREG,SREG
		push	TEMP_0
		push	XH
		push	XL	
		
		in		TEMP_0,UDR					; erstmal Daten holen, notfalls wegwerfen...
		             
		sbrc	FLAGS,FLAG_RX_OK			; alte Daten schon bearbeitet?
		rjmp	irq_rx_end					; nein, also ignorieren

		cpi		TEMP_0,STB					; Start-Byte?
		breq	irq_rx_end

		cpi		TEMP_0,ETB					; Ende-Byte?
		brne	irq_rx_data

		sbrs	FLAGS,FLAG_RX_END			; Ende erwartet?
		rjmp	irq_rx_sync_err				; nein...
		cbr		FLAGS,(1<<FLAG_RX_END)
		rjmp	irq_rx_end
		
irq_rx_data:
		load_p	X,RX_BUF					; Empfangsbuffer
   		st		X,TEMP_0   					; Speichern im SRAM und Eingabeadresse in X erhhen
		sbr		FLAGS,(1<<FLAG_RX_END) + (1<<FLAG_RX_OK)		; jetzt auf Ende-Byte warten
		rjmp	irq_rx_end

irq_rx_sync_err:
		cbr		FLAGS,(1<<FLAG_RX_END) + (1<<FLAG_RX_OK)		; Flags lschen
		
irq_rx_end:
		pop		XL
		pop		XH
		pop		TEMP_0
		out		SREG,SICH_SREG

   		reti

.include "delay.inc"
.include "dfue.inc"
.include "auswert.inc"



  
;**** The Quelltext ends here *************************************************
