
;***************************************************************************
;*
;* "LIN_Frames"
;*
;* This subroutine generates the required Lin Frames for
;* transmitting. 
;* 
;*
;***************************************************************************

;**** Wake up Frame ********************************************************
h_send_wf:
//		load_p	Z,(2*text_wf)
//		rcall	uart_send_text
		cbr		FLAGS,(1<<UF)			; 	
		ldi     ZL, LOW(lin_data_wf*2)
		ldi     ZH, HIGH(lin_data_wf*2)            	; Z-Pointer laden
		ldi		YL,LOW(Buffer)
		ldi		YH,HIGH(Buffer)
		clr		BYTECNT				; leeren
laden_uf:		
		lpm		TEMP_1,Z+			;in den Stack
		cpi		TEMP_1,0x99			;Ende d. Frames?
		breq	weiter_uf
		st		Y+,TEMP_1
		inc		BYTECNT				;mitzhlen
		rjmp	laden_uf
weiter_uf:
		rcall	send_frame
		ret

;**** Slave Frame **********************************************************


h_send_sf:
//		load_p	Z,(2*text_sf)
//		rcall	uart_send_text
		cbr		FLAGS,(1<<SF)					; Anf aus
		ldi		YL,LOW(Buffer)
		ldi		YH,HIGH(Buffer)
			
		clr		BYTECNT							; leeren
			
		ldi		TEMP_1,ADR_SL					; 0xE2 Slave Adresse
		st		Y+,TEMP_1				
		inc		BYTECNT
		rcall 	send_frame


		ldi		YL,LOW(RXBuffer)
		ldi		YH,HIGH(RXBuffer)
		ldi		TEMP_1,0x05						; 6 Bytes Response erwartet, das 4. Byte ist interessant...		
		clr		TXBYTE
empf:
		inc		BYTECNT
		rcall  	getchar
		tst		BITCNT							; Timeout ?
		breq	empf_weiter
		rjmp	timeout_sf						; 
    empf_weiter:		
		st		Y+, TXBYTE
		cp		BYTECNT,TEMP_1					; 6. Byte Antwort da?
		brlo	empf
		
		
		clr		BYTECNT							; leeren
		sbr		FLAGS,(1<<LRX)
		ret										; Werte in Buffer
	timeout_sf:
//		load_p	Z,(2*text_ta)
//		rcall	uart_send_text
		clr		BYTECNT							; leeren
		ret

;**** Master Frame *********************************************************************************

h_send_mf:
//		load_p	Z,(2*text_mf)
//		rcall	uart_send_text	
		cbr			FLAGS,(1<<MF)				; Anf aus
		clr			TEMP_3						; Prfsummenberechnung
		clr			TEMP_2
		clr			BYTECNT						; leeren
		inc			MFCNT						; Masterframecnt erhhen
		mov			TEMP_2,MFCNT
		cpi			TEMP_2,0x10
		brlo		test_start_mf				; kleiner als 16

		clr			MFCNT
		

test_start_mf:

		sbrs		FLAGS,START					; Startframes?
		rjmp		weiter_mfc					; nein
		dec			STARTCNT
		breq		start_komplett
		ldi			TEMP_2,20
		cp			STARTCNT, TEMP_2
		brge		start_phase_1
		
		ldi    	ZL, LOW(lin_data_mfn*2)			; Zeiger auf MF  Start 2 Daten
		ldi    	ZH, HIGH(lin_data_mfn*2) 		; Z-Pointer laden
		clr			TEMP_2
		rjmp		weiter_frame

start_phase_1:
		ldi    	ZL, LOW(lin_data_mfs*2)			; Zeiger auf MF  Start 1 Daten
		ldi    	ZH, HIGH(lin_data_mfs*2) 		; Z-Pointer laden
		clr			TEMP_2
		rjmp		weiter_frame		

start_komplett:
		cbr			FLAGS,(1<<START)			; Startflag lschen
weiter_mfc:		
		clr			TEMP_2
		ldi    	ZL, LOW(lin_data_mf*2)			; Zeiger auf MF Daten
		ldi    	ZH, HIGH(lin_data_mf*2) 		; Z-Pointer laden

weiter_frame:	
		ldi		YL,LOW(Buffer)
		ldi		YH,HIGH(Buffer)
		
		
byte_1:
		ldi		TEMP_0,ADR_MA						; Addresse Masterframe
		st		Y+,TEMP_0
		inc		BYTECNT
byte_2:
		mov		TEMP_0,MFCNT	
		add  	TEMP_3,TEMP_0						; f. Prfsumme
		st		Y+,TEMP_0
		inc		BYTECNT

		ldi		TEMP_0, 0x99					; Endmarkierung laden
byte3_6:
		lpm		TEMP_1,Z+	
		cp		TEMP_1, TEMP_0
		breq	byte_ps
		st		Y+,TEMP_1
		add		TEMP_3,TEMP_1					; Prfsumme
		adc		TEMP_2,TEMP_2
		add		TEMP_3,TEMP_2
		clr		TEMP_2
		inc		BYTECNT
		rjmp	byte3_6


byte_ps:
		com		TEMP_3							; Einerkomplement = Prfsumme
		st		Y+,TEMP_3						; Prfsumme Classic
		inc		BYTECNT
		
		clr		TEMP_3
		st		Y+,TEMP_3						; nullen am Ende
		inc		BYTECNT



;**** Frame senden **************************************************************

send_frame:
		cli										; Interrupts sperren
		inc		FRAMECNT						; Frame senden, cnt 1+
		ldi		YL,LOW(Buffer)
		ldi		YH,HIGH(Buffer)					; Header senden
		ldi		TXBYTE,SYNC						;
		ldi		TEMP_3,SYNC_BR					; SyncBreak
		mov		BITCNT, TEMP_3
		cbi		PORTB,S_TxD						; send a 0(dominant)
		nop
sync_brk:
		rcall	UART_delay						; One bit delay
		rcall 	UART_delay

		dec		BITCNT							; If not all bit sent
		brne	sync_brk						; send next
		sbi		PORTB,S_TxD						; send a 1(Rezessiv)
		rcall 	UART_delay						; 1 bit Delimiter
		rcall 	UART_delay
			
		rcall  	putchar
send_frame_1:
		ld		TXBYTE,Y+
		dec		BYTECNT
		rcall  	putchar
		tst		BYTECNT							; Ende d. Frames?
		breq    frame_end						; Ja, weiter
		rjmp    send_frame_1

frame_end:		
		rcall	UART_delay						; Inter Frame Space
		rcall 	UART_delay
		clr		BYTECNT
		sei										; Interrupts wieder frei
		ret






;***************************************************************************
;*
;* "putchar"
;*
;* This subroutine transmits the byte stored in the "Txbyte" register
;* The number of stop bits used is set with the sb constant
;*
;* Number of words	:14 including return
;* Number of cycles	:Depens on bit rate
;* Low registers used	:None
;* High registers used	:2 (TEMP_3,Txbyte)
;* Low registers used	:1 (bitcnt)
;* Pointers used	:None
;*
;***************************************************************************


putchar:	
		ldi			TEMP_3,9+sb			;1+8+sb (sb is # of stop bits)
		mov			BITCNT,TEMP_3
		com			TXBYTE				;Inverte everything
		sec								;Start bit

putchar0:	
		brcc		putchar1			;If carry set
		cbi			PORTB,S_TxD			;send a '0'
		rjmp		putchar2			;else	

putchar1:	
		sbi			PORTB,S_TxD			;send a '1'
		nop

putchar2:
		rcall 		UART_delay			;One bit delay
		rcall 		UART_delay

		lsr			TXBYTE				;Get next bit
		dec			BITCNT				;If not all bit sent
		brne		putchar0			;send next
								;else
		ret						;return


;***************************************************************************
;*
;* "getchar"
;*
;* This subroutine receives one byte and returns it in the "Rxbyte" register
;*
;* Number of words	:14 including return
;* Number of cycles	:Depens on when data arrives
;* Low registers used	:None
;* High registers used	:2 (bitcnt,Rxbyte)
;* Pointers used	:None
;*
;***************************************************************************

getchar:	
		ldi 		TEMP_3,9			;8 data bit + 1 stop bit
		mov			BITCNT,TEMP_3
		clr			COUNTER				;counter leeren fr Timeout
getchar1:	
		sbrc		COUNTER,0			;wenn 10ms nichts passiert ist..
		ret								;Timeout
		sbic 		PINB,S_RxD			;Wait for start bit
		rjmp 		getchar1

		rcall 		UART_delay			;0.5 bit delay

getchar2:	
		rcall 		UART_delay			;1 bit delay
		rcall 		UART_delay		

		clc								;clear carry
		sbic 		PINB,S_RxD			;if RX pin high
		sec						;

		dec 		BITCNT				;If bit is stop bit
		breq 		getchar3			;return
							        	;else
		ror 		TXBYTE				;shift bit into Rxbyte
		rjmp 		getchar2			;go get next

getchar3:	
		ret


;***************************************************************************
;*
;* "UART_delay"
;*
;* This delay subroutine generates the required delay between the bits when
;* transmitting and receiving bytes. The total execution time is set by the
;* constant "b":
;*
;*	3b + 7 cycles (including rcall and ret)
;*
;* Number of words	:4 including return
;* Low registers used	:None
;* High registers used	:1 (temp)
;* Pointers used	:None
;*
;***************************************************************************

UART_delay:	
		ldi		TEMP_0,b

UART_delay1:
		dec		TEMP_0
		brne	UART_delay1

		ret
	
 
